/*
 * This file is part of X2C. http://x2c.lcm.at/
 * $LastChangedRevision: 1603 $
 */
/* USERCODE-BEGIN:Description                                                                                         */
/* Description: */
/* USERCODE-END:Description                                                                                           */
#include "ConvFnc_Sin2Limiter_FiP16.h"

/* USERCODE-BEGIN:PreProcessor                                                                                        */
/* USERCODE-END:PreProcessor                                                                                          */

extern uint8 MaskParamBuffer[];
extern uint8 ImplParamBuffer[];
extern uint8 SaveFncDataBuffer[];

/** Implementation parameter data structure */
typedef struct {
    int32 RateUp;
    int32 RateDown;
} SIN2LIMITER_FIP16_IMPL_PARAM;

/** Private prototypes */
static uint8 convertM2I(const SIN2LIMITER_FIP16_MASK_PARAM *maskParam, SIN2LIMITER_FIP16_IMPL_PARAM *impParam, float_CoT ts);

/**
 * @brief Converts Mask parameters to Implementation parameters.
 *
 * @param[in] maskParam Mask parameters
 * @param[out] impParam Implementation parameters
 * @param[in] ts Block sample time
 *
 * @return Error: zero on success, not zero in case of conversion error
 */
static uint8 convertM2I(const SIN2LIMITER_FIP16_MASK_PARAM *maskParam, SIN2LIMITER_FIP16_IMPL_PARAM *impParam, float_CoT ts)
{
    uint8 error = (uint8)0;
/* USERCODE-BEGIN:Conversion                                                                                          */
    float_CoT mp_Tr, mp_Tf;

    mp_Tr = maskParam->Tr;
    mp_Tf = maskParam->Tf;

    /* check range of rising/falling time */
    if (mp_Tr < ts)
    {
        mp_Tr = ts;
    }
    if (mp_Tf < ts)
    {
        mp_Tf = ts;
    }

    impParam->RateUp = getQx32Value(ts/mp_Tr, 31);
    impParam->RateDown = getQx32Value(ts/mp_Tf, 31);

/* USERCODE-END:Conversion                                                                                            */
    return (error);
}

/**
 * @brief Load block mask parameter data.
 *
 * @param[in] maskParam Mask parameter data structure
 * @param[out] data Data
 * @param[out] dataLen Data length
 * @param[in] maxSize Maximum Service data length
 *
 * @return Error: zero on success, not zero in case of load- or conversion error
 */
uint8 Sin2Limiter_FiP16_LoadMP(const SIN2LIMITER_FIP16_MASK_PARAM *maskParam, uint8 data[], uint16* dataLen, uint16 maxSize)
{
    uint8 error = (uint8)0;
    if ((uint16)24 > maxSize)
    {
        error = (uint8)1;
    }
    else
    {
        data[0] = (uint8)((*(uint64*)&(maskParam->Tr)) & 0x00000000000000FF);
        data[1] = (uint8)((*(uint64*)&(maskParam->Tr) >> 8) & 0x00000000000000FF);
        data[2] = (uint8)((*(uint64*)&(maskParam->Tr) >> 16) & 0x00000000000000FF);
        data[3] = (uint8)((*(uint64*)&(maskParam->Tr) >> 24) & 0x00000000000000FF);
        data[4] = (uint8)((*(uint64*)&(maskParam->Tr) >> 32) & 0x00000000000000FF);
        data[5] = (uint8)((*(uint64*)&(maskParam->Tr) >> 40) & 0x00000000000000FF);
        data[6] = (uint8)((*(uint64*)&(maskParam->Tr) >> 48) & 0x00000000000000FF);
        data[7] = (uint8)((*(uint64*)&(maskParam->Tr) >> 56) & 0x00000000000000FF);
        data[8] = (uint8)((*(uint64*)&(maskParam->Tf)) & 0x00000000000000FF);
        data[9] = (uint8)((*(uint64*)&(maskParam->Tf) >> 8) & 0x00000000000000FF);
        data[10] = (uint8)((*(uint64*)&(maskParam->Tf) >> 16) & 0x00000000000000FF);
        data[11] = (uint8)((*(uint64*)&(maskParam->Tf) >> 24) & 0x00000000000000FF);
        data[12] = (uint8)((*(uint64*)&(maskParam->Tf) >> 32) & 0x00000000000000FF);
        data[13] = (uint8)((*(uint64*)&(maskParam->Tf) >> 40) & 0x00000000000000FF);
        data[14] = (uint8)((*(uint64*)&(maskParam->Tf) >> 48) & 0x00000000000000FF);
        data[15] = (uint8)((*(uint64*)&(maskParam->Tf) >> 56) & 0x00000000000000FF);
        data[16] = (uint8)((*(uint64*)&(maskParam->ts_fact)) & 0x00000000000000FF);
        data[17] = (uint8)((*(uint64*)&(maskParam->ts_fact) >> 8) & 0x00000000000000FF);
        data[18] = (uint8)((*(uint64*)&(maskParam->ts_fact) >> 16) & 0x00000000000000FF);
        data[19] = (uint8)((*(uint64*)&(maskParam->ts_fact) >> 24) & 0x00000000000000FF);
        data[20] = (uint8)((*(uint64*)&(maskParam->ts_fact) >> 32) & 0x00000000000000FF);
        data[21] = (uint8)((*(uint64*)&(maskParam->ts_fact) >> 40) & 0x00000000000000FF);
        data[22] = (uint8)((*(uint64*)&(maskParam->ts_fact) >> 48) & 0x00000000000000FF);
        data[23] = (uint8)((*(uint64*)&(maskParam->ts_fact) >> 56) & 0x00000000000000FF);
        *dataLen = (uint16)24;
    }
    return (error);
}

/**
 * @brief Save block mask parameter data.
 *
 * @param[in] block Pointer to block structure
 * @param[out] maskParam Mask parameter data structure
 * @param[in] data Data
 * @param dataLen Length of mask parameter data stream
 *
 * @return Error: zero on success, not zero in case of save- or conversion error
 */
uint8 Sin2Limiter_FiP16_SaveMP(SIN2LIMITER_FIP16 *block, SIN2LIMITER_FIP16_MASK_PARAM *maskParam, const uint8 data[], uint16 dataLen)
{
    uint8 error = (uint8)0;

    if (dataLen != (uint16)24)
    {
        error = (uint8)1;
    }
    else
    {
        uint64 tmp64;
        float_CoT block_ts;
        SIN2LIMITER_FIP16_IMPL_PARAM *implParam = (SIN2LIMITER_FIP16_IMPL_PARAM*)ImplParamBuffer;

        /** - Cache old mask parameters */
        memcpy(MaskParamBuffer, maskParam, sizeof(*maskParam));

        /** - Save new mask parameters */
        tmp64 = (uint64)data[0] + \
            ((uint64)data[1] << 8) + ((uint64)data[2] << 16) + \
            ((uint64)data[3] << 24) + ((uint64)data[4] << 32) + \
            ((uint64)data[5] << 40) + ((uint64)data[6] << 48) + \
            ((uint64)data[7] << 56);
        maskParam->Tr = (float64)(*(float64*)&tmp64);
        tmp64 = (uint64)data[8] + \
            ((uint64)data[9] << 8) + ((uint64)data[10] << 16) + \
            ((uint64)data[11] << 24) + ((uint64)data[12] << 32) + \
            ((uint64)data[13] << 40) + ((uint64)data[14] << 48) + \
            ((uint64)data[15] << 56);
        maskParam->Tf = (float64)(*(float64*)&tmp64);
        tmp64 = (uint64)data[16] + \
            ((uint64)data[17] << 8) + ((uint64)data[18] << 16) + \
            ((uint64)data[19] << 24) + ((uint64)data[20] << 32) + \
            ((uint64)data[21] << 40) + ((uint64)data[22] << 48) + \
            ((uint64)data[23] << 56);
        maskParam->ts_fact = (float64)(*(float64*)&tmp64);

        /** - Get Block Sample Time */
        block_ts = maskParam->ts_fact * getModelSampleTime();
        /** - Convert mask parameters to implementation parameters */
        error = convertM2I(maskParam, implParam, block_ts);

        /** - Prepare implementation parameters for save function */
        SaveFncDataBuffer[0] = (uint8)(implParam->RateUp & 0x000000FF);
        SaveFncDataBuffer[1] = (uint8)((implParam->RateUp >> 8) & 0x000000FF);
        SaveFncDataBuffer[2] = (uint8)((implParam->RateUp >> 16) & 0x000000FF);
        SaveFncDataBuffer[3] = (uint8)((implParam->RateUp >> 24) & 0x000000FF);
        SaveFncDataBuffer[4] = (uint8)(implParam->RateDown & 0x000000FF);
        SaveFncDataBuffer[5] = (uint8)((implParam->RateDown >> 8) & 0x000000FF);
        SaveFncDataBuffer[6] = (uint8)((implParam->RateDown >> 16) & 0x000000FF);
        SaveFncDataBuffer[7] = (uint8)((implParam->RateDown >> 24) & 0x000000FF);

        /** - Execute save function of block */
        error = Sin2Limiter_FiP16_Save(block, SaveFncDataBuffer, 8);

        /** - Check for errors during execution of save function */
        if (error != 0)
        {
            /* Restore old mask parameter */
            memcpy(maskParam, MaskParamBuffer, sizeof(*maskParam));
        }

    }
    return (error);
}

/**
 * @brief Initializes Implementation parameters from Mask parameters.
 *
 * @param[out] block Block
 * @param[in] maskParam Mask parameters
 *
 * @return Error: zero on success, not zero in case of initialization error
 */
uint8 Sin2Limiter_FiP16_InitMP(SIN2LIMITER_FIP16 *block, const SIN2LIMITER_FIP16_MASK_PARAM *maskParam)
{
    uint8 error = (uint8)0;
    float_CoT block_ts;

    SIN2LIMITER_FIP16_IMPL_PARAM *implParam = (SIN2LIMITER_FIP16_IMPL_PARAM*)ImplParamBuffer;

    /** - Get Block Sample Time */
    block_ts = maskParam->ts_fact * getModelSampleTime();
    /** - Convert mask parameters to implementation parameters */
    error = convertM2I(maskParam, implParam, block_ts);

    /** - Prepare implementation parameters for save function */
    SaveFncDataBuffer[0] = (uint8)(implParam->RateUp & 0x000000FF);
    SaveFncDataBuffer[1] = (uint8)((implParam->RateUp >> 8) & 0x000000FF);
    SaveFncDataBuffer[2] = (uint8)((implParam->RateUp >> 16) & 0x000000FF);
    SaveFncDataBuffer[3] = (uint8)((implParam->RateUp >> 24) & 0x000000FF);
    SaveFncDataBuffer[4] = (uint8)(implParam->RateDown & 0x000000FF);
    SaveFncDataBuffer[5] = (uint8)((implParam->RateDown >> 8) & 0x000000FF);
    SaveFncDataBuffer[6] = (uint8)((implParam->RateDown >> 16) & 0x000000FF);
    SaveFncDataBuffer[7] = (uint8)((implParam->RateDown >> 24) & 0x000000FF);

    /** - Execute save function of block */
    error = Sin2Limiter_FiP16_Save(block, SaveFncDataBuffer, 8);

    return (error);
}
